@@ -34,69 +34,75 @@ static gboolean
 coded_buffer_create (GstVaapiCodedBuffer * buf, guint buf_size,
     GstVaapiContext * context)
 {
-  GstVaapiDisplay *const display = GST_VAAPI_OBJECT_DISPLAY (buf);
+  GstVaapiDisplay *const display = GST_VAAPI_CODED_BUFFER_DISPLAY (buf);
   VABufferID buf_id;
   gboolean success;
 
   GST_VAAPI_DISPLAY_LOCK (display);
   success = vaapi_create_buffer (GST_VAAPI_DISPLAY_VADISPLAY (display),
-      GST_VAAPI_OBJECT_ID (context), VAEncCodedBufferType, buf_size, NULL,
-      &buf_id, NULL);
+      GST_VAAPI_CONTEXT_ID (context), VAEncCodedBufferType, buf_size,
+      NULL, &buf_id, NULL);
   GST_VAAPI_DISPLAY_UNLOCK (display);
   if (!success)
     return FALSE;
 
   GST_DEBUG ("coded buffer %" GST_VAAPI_ID_FORMAT, GST_VAAPI_ID_ARGS (buf_id));
-  GST_VAAPI_OBJECT_ID (buf) = buf_id;
+  GST_VAAPI_CODED_BUFFER_ID (buf) = buf_id;
   return TRUE;
 }
 
 static void
-coded_buffer_destroy (GstVaapiCodedBuffer * buf)
+coded_buffer_free (GstVaapiCodedBuffer * buf)
 {
-  GstVaapiDisplay *const display = GST_VAAPI_OBJECT_DISPLAY (buf);
+  GstVaapiDisplay *const display = GST_VAAPI_CODED_BUFFER_DISPLAY (buf);
   VABufferID buf_id;
 
-  buf_id = GST_VAAPI_OBJECT_ID (buf);
+  buf_id = GST_VAAPI_CODED_BUFFER_ID (buf);
   GST_DEBUG ("coded buffer %" GST_VAAPI_ID_FORMAT, GST_VAAPI_ID_ARGS (buf_id));
 
   if (buf_id != VA_INVALID_ID) {
     GST_VAAPI_DISPLAY_LOCK (display);
     vaapi_destroy_buffer (GST_VAAPI_DISPLAY_VADISPLAY (display), &buf_id);
     GST_VAAPI_DISPLAY_UNLOCK (display);
-    GST_VAAPI_OBJECT_ID (buf) = VA_INVALID_ID;
+    GST_VAAPI_CODED_BUFFER_ID (buf) = VA_INVALID_ID;
   }
+
+  gst_vaapi_display_replace (&GST_VAAPI_CODED_BUFFER_DISPLAY (buf), NULL);
+
+  g_free (buf);
 }
 
 static gboolean
 coded_buffer_map (GstVaapiCodedBuffer * buf)
 {
+  GstVaapiDisplay *const display = GST_VAAPI_CODED_BUFFER_DISPLAY (buf);
+
   if (buf->segment_list)
     return TRUE;
 
-  GST_VAAPI_OBJECT_LOCK_DISPLAY (buf);
-  buf->segment_list = vaapi_map_buffer (GST_VAAPI_OBJECT_VADISPLAY (buf),
-      GST_VAAPI_OBJECT_ID (buf));
-  GST_VAAPI_OBJECT_UNLOCK_DISPLAY (buf);
+  GST_VAAPI_DISPLAY_LOCK (display);
+  buf->segment_list =
+      vaapi_map_buffer (GST_VAAPI_DISPLAY_VADISPLAY (display),
+      GST_VAAPI_CODED_BUFFER_ID (buf));
+  GST_VAAPI_DISPLAY_UNLOCK (display);
   return buf->segment_list != NULL;
 }
 
 static void
 coded_buffer_unmap (GstVaapiCodedBuffer * buf)
 {
+  GstVaapiDisplay *const display = GST_VAAPI_CODED_BUFFER_DISPLAY (buf);
+
   if (!buf->segment_list)
     return;
 
-  GST_VAAPI_OBJECT_LOCK_DISPLAY (buf);
-  vaapi_unmap_buffer (GST_VAAPI_OBJECT_VADISPLAY (buf),
-      GST_VAAPI_OBJECT_ID (buf), (void **) &buf->segment_list);
-  GST_VAAPI_OBJECT_UNLOCK_DISPLAY (buf);
+  GST_VAAPI_DISPLAY_LOCK (display);
+  vaapi_unmap_buffer (GST_VAAPI_DISPLAY_VADISPLAY (display),
+      GST_VAAPI_CODED_BUFFER_ID (buf), (void **) &buf->segment_list);
+  GST_VAAPI_DISPLAY_UNLOCK (display);
 }
 
-/* *INDENT-OFF* */
-#define gst_vaapi_coded_buffer_finalize coded_buffer_destroy
-GST_VAAPI_OBJECT_DEFINE_CLASS (GstVaapiCodedBuffer, gst_vaapi_coded_buffer)
-/* *INDENT-ON* */
+GST_DEFINE_MINI_OBJECT_TYPE (GstVaapiCodedBuffer, gst_vaapi_coded_buffer);
 
 /*
  * gst_vaapi_coded_buffer_new:
@@ -117,13 +123,21 @@ gst_vaapi_coded_buffer_new (GstVaapiContext * context, guint buf_size)
   g_return_val_if_fail (context != NULL, NULL);
   g_return_val_if_fail (buf_size > 0, NULL);
 
-  display = GST_VAAPI_OBJECT_DISPLAY (context);
+  display = GST_VAAPI_CONTEXT_DISPLAY (context);
   g_return_val_if_fail (display != NULL, NULL);
 
-  buf = gst_vaapi_object_new (gst_vaapi_coded_buffer_class (), display);
+  buf = g_new (GstVaapiCodedBuffer, 1);
   if (!buf)
     return NULL;
 
+  gst_mini_object_init (GST_MINI_OBJECT_CAST (buf), 0,
+      GST_TYPE_VAAPI_CODED_BUFFER, NULL, NULL,
+      (GstMiniObjectFreeFunction) coded_buffer_free);
+
+  GST_VAAPI_CODED_BUFFER_DISPLAY (buf) = gst_object_ref (display);
+  GST_VAAPI_CODED_BUFFER_ID (buf) = VA_INVALID_ID;
+  buf->segment_list = NULL;
+
   if (!coded_buffer_create (buf, buf_size, context))
     goto error;
   return buf;
@@ -131,7 +145,7 @@ gst_vaapi_coded_buffer_new (GstVaapiContext * context, guint buf_size)
   /* ERRORS */
 error:
   {
-    gst_vaapi_object_unref (buf);
+    gst_vaapi_coded_buffer_unref (buf);
     return NULL;
   }
 }
