@@ -249,7 +249,7 @@ GstVaapiGLApi
 gl_get_curent_api_once ()
 {
   static GstVaapiGLApi cur_api = GST_VAAPI_GL_API_NONE;
-  static volatile gsize _init = 0;
+  static gsize _init = 0;
 
   if (g_once_init_enter (&_init)) {
     cur_api = gl_get_current_api (NULL, NULL);
@@ -385,8 +385,8 @@ gst_vaapi_texture_glx_put_surface_unlocked (GstVaapiTexture * texture,
   }
 
   flags = GST_MINI_OBJECT_FLAGS (texture);
-  txc = g_texcoords[! !(flags & GST_VAAPI_TEXTURE_ORIENTATION_FLAG_X_INVERTED)];
-  tyc = g_texcoords[! !(flags & GST_VAAPI_TEXTURE_ORIENTATION_FLAG_Y_INVERTED)];
+  txc = g_texcoords[!!(flags & GST_VAAPI_TEXTURE_ORIENTATION_FLAG_X_INVERTED)];
+  tyc = g_texcoords[!!(flags & GST_VAAPI_TEXTURE_ORIENTATION_FLAG_Y_INVERTED)];
 
   glColor4f (1.0f, 1.0f, 1.0f, 1.0f);
   glBegin (GL_QUADS);
